/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Commando Interpreter
------------------------------------------------------------------------------*/

#include "cmd.h"

volatile unsigned int variable[MAX_VAR];
char cmd_buffer[40];

CMD_ITEM CMD_TABLE[] = // Befehls-Tabelle
{
	{"RESET",command_reset},
	{"ARP",command_arp},
	{"TCP",command_tcp},
	{"IP",command_ip},
	{"MAC",command_mac},
	{"VER",command_ver},
	{"SV",command_setvar},
	{"TIME",command_time},
	{NULL,NULL} 
};

//------------------------------------------------------------------------------
//Zeichenprfung auf Trennzeichen
unsigned char is_space (char ch)
{
	
	return (ch == ' ' || ch == ',' || ch == '.');
}

//------------------------------------------------------------------------------
//Nchstes Trennzeichen suchen und String abschliessen
char* cmd_skip_chars(char *sptr)
{
	while (*sptr && !is_space(*sptr)) sptr++;
	if (*sptr) // Wenn noch kein Stringende erreicht ist,
		*sptr++ = 0; // dann String mit 0 abschliessen und Zeiger erhhen
	return sptr;
}

//------------------------------------------------------------------------------
//Commando auswerten
unsigned char cmd_exec(char *pstr)
{
	//Stringzeiger;
	char *ps;
	unsigned char cmd_index = 0; // Befehlsindex
 
	//Anfangsposition von Token in ps speichern
	ps = pstr;  
	//Stringende suchen und abschliessen
	pstr = cmd_skip_chars(pstr);

	usart_write("\r\n");
	//Stringvergleich mit Befehlsliste durchfhren
	while (CMD_TABLE[cmd_index].cmd && strcasecmp(CMD_TABLE[cmd_index].cmd,ps)) cmd_index++;
	// Wenn Befehlsindex zu gross, dann mit Fehler beenden
	if (!CMD_TABLE[cmd_index].cmd) return(0);  
	
	for (unsigned char a = 0; a<MAX_VAR;a++)
	{
		ps = pstr;
		pstr = cmd_skip_chars(pstr);
		variable[a] = atoi(ps);
	}

	CMD_TABLE[cmd_index].fp(); // Befehl ausfhren und Ergebnis zurckgeben
	return(1); 
}

//------------------------------------------------------------------------------
//Reset ausfhren
void command_reset (void)
{
	RESET();
}

//------------------------------------------------------------------------------
//print own IP
void command_ip (void)
{
	if (*((unsigned int*)&variable[0]) != 0x00000000)
	{
		myip[0] = variable[0];
		myip[1] = variable[1];
		myip[2] = variable[2];
		myip[3] = variable[3];
		
		//IP aus EEPROM auslesen
		for (unsigned char count = 0; count<4;count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(IP_EEPROM_STORE+count),myip[count]);
		}
	}
	usart_write("My IP: %1i.%1i.%1i.%1i\r\n",myip[0],myip[1],myip[2],myip[3]);
}

//------------------------------------------------------------------------------
//print own mac
void command_mac (void)
{
	usart_write("My MAC: %2x:%2x:%2x:%2x:%2x:%2x\r\n",mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);
}

//------------------------------------------------------------------------------
//print enc28j60 chip version
void command_ver (void)
{
#if USE_ENC28J60
	usart_write("ENC28J60-Version: %1x\r\n", enc28j60Read(EREVID));
#endif

#if USE_RTL8019
	usart_write("RTL8019 Ethernetcard\r\n");
#endif
}

//------------------------------------------------------------------------------
//print ARP table
void command_arp (void)
{
	for (unsigned char index = 0;index<MAX_ARP_ENTRY;index++)
	{
		usart_write("%2i  MAC:%2x",index,arp_entry[index].arp_t_mac[0]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[1]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[2]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[3]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[4]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[5]);
		
		usart_write("  IP:%3i",(arp_entry[index].arp_t_ip&0x000000FF));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0x0000FF00)>>8));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0x00FF0000)>>16));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0xFF000000)>>24));
			
		usart_write("  Time:%4i\r\n",arp_entry[index].arp_t_time);
	}
}

//------------------------------------------------------------------------------
//print ARP table
void command_tcp (void)
{
	for (unsigned char index = 0;index<MAX_TCP_ENTRY;index++)
	{
		usart_write("%2i",index);
		usart_write("  IP:%3i",(tcp_entry[index].ip&0x000000FF));
		usart_write(".%3i",((tcp_entry[index].ip&0x0000FF00)>>8));
		usart_write(".%3i",((tcp_entry[index].ip&0x00FF0000)>>16));
		usart_write(".%3i",((tcp_entry[index].ip&0xFF000000)>>24));
		usart_write("  PORT:%4i",LBBL_ENDIAN_INT(tcp_entry[index].src_port));
		usart_write("  Time:%4i\r\n",tcp_entry[index].time);
	}
}

//------------------------------------------------------------------------------
//ndern einer Variable
void command_setvar (void)
{
	var_array[variable[0]] = variable[1];
	usart_write("Inhalt der Variable[%2i] = %2i\r\n",variable[0],var_array[variable[0]]);
}

//------------------------------------------------------------------------------
//print Time
void command_time (void)
{
	unsigned char hh = (time/3600)%24;
	unsigned char mm = (time/60)%60;
	unsigned char ss = time %60;
	usart_write ("\n\rTIME: %2i:%2i:%2i\r\n",hh,mm,ss);
}







